﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;



// neu
using Microsoft.Win32;// opendialog


using System.Windows.Markup;
using System.Windows.Xps;  // Verweis: System.Printing
using System.Windows.Xps.Packaging;  // Verweis: ReachFramework
using System.IO;
using System.Diagnostics;
using System.Windows.Controls.Primitives;



namespace CAD
{
    /// <summary>
    /// Interaktionslogik für MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window, IChangeParent, IConsole
    {
        private bool bStart = true;
        MyCanvas canvas;

        private Color colorLine = Colors.Black;
        private Color colorFill = Colors.Red;
        private DashStyle lineStyle=DashStyles.Solid;
        private double linewidth = 1.0;

        private CADParent _copyPasteElement = null;

        public MainWindow()
        {
            InitializeComponent();
            bStart = true;
            setGUI();
        }

        private void setGUI()
        {
            Height = 800;
            Width = 1000;

            BnArrowNone.IsChecked = true;
            
            canvas = new MyCanvas(this);            
            gridcanvas1.Content = canvas;

            bStart = false;

            // Labor2: CheckCanExecuted
            CommandBindings.Add(new CommandBinding(MyCommands.Newfile, NewExecuted));
            CommandBindings.Add(new CommandBinding(MyCommands.Openfile, OpenExecuted));

            CommandBindings.Add(new CommandBinding(MyCommands.Savefile, SaveExecuted, CheckCanExecuted));
            CommandBindings.Add(new CommandBinding(MyCommands.Export2DXFfile, Export2DXFfile, CheckCanExecuted));

            CommandBindings.Add(new CommandBinding(MyCommands.Copy, CopyElement, CheckCanExecuted));
            CommandBindings.Add(new CommandBinding(MyCommands.Paste, PasteElement, CheckCanExecuted));
            CommandBindings.Add(new CommandBinding(MyCommands.Delete, DeleteElement, CheckCanExecuted));

            CommandBindings.Add(new CommandBinding(MyCommands.PrintElements, PrintElements, CheckCanExecuted));
            CommandBindings.Add(new CommandBinding(MyCommands.Console, InsertPerConsole));

            /*CADLine line1;
            line1 = new CADLine(10, 20, 100, 20);  // in mm
            line1.StrokeThickness = 4;
            line1.BrushLine = Colors.Aqua;
            Middleware.getInstance().cadListe.Add(line1);
            graphicList.Items.Add(line1);

            line1 = new CADLine(110, 120, 10, 220);  // in mm
            line1.StrokeThickness = 3;
            line1.BrushLine = Colors.Red;
            Middleware.getInstance().cadListe.Add(line1);
            graphicList.Items.Add(line1);*/
        }  // setGUI


        // Labor2: CheckCanExecuted
        private void CheckCanExecuted(object sender, CanExecuteRoutedEventArgs e)
        {
            e.CanExecute = Middleware.getInstance().cadListe.Count > 0;
        }

        // Labor2
        private void clearAllButton(ToggleButton bnActive, CADParent.GrfTypen gtyp)
        {
            BnArrowNone.IsChecked = false;
            BnLine.IsChecked = false;  
            BnHoriLine.IsChecked = false;
            BnVertLine.IsChecked = false;
            BnEllipse.IsChecked = false;
            BnRectangle.IsChecked = false;
            BnCircle.IsChecked = false;
            BnText.IsChecked = false;

            bnActive.IsChecked = true;
            canvas.Modus = gtyp;
            canvas.MoveFirst = 0;
        }

        // Labor2
        private void BnArrowNone_Click(object sender, RoutedEventArgs e)
        {
            clearAllButton(BnArrowNone, CADParent.GrfTypen.C_NONE);
        }

        // Labor2
        private void BnLine_Click(object sender, RoutedEventArgs e)
        {
            clearAllButton(BnLine, CADParent.GrfTypen.C_LINE);
        }

        private void BnHoriLine_Click(object sender, RoutedEventArgs e)
        {
            clearAllButton(BnHoriLine, CADParent.GrfTypen.C_HORILINE);
        }


        private void BnVertLine_Click(object sender, RoutedEventArgs e)
        {
            clearAllButton(BnVertLine, CADParent.GrfTypen.C_VERTLINE);
        }
      

        // Labor2
        private void BnRectangle_Click(object sender, RoutedEventArgs e)
        {
            clearAllButton(BnRectangle, CADParent.GrfTypen.C_RECTANGLE);
        }

        // Labor2
        private void BnCircle_Click(object sender, RoutedEventArgs e)
        {
            clearAllButton(BnCircle, CADParent.GrfTypen.C_CIRCLE);
        }

        private void BnEllipse_Click(object sender, RoutedEventArgs e)
        {
            clearAllButton(BnEllipse, CADParent.GrfTypen.C_ELLIPSE);
        }


        // Labor2
        private void BnText_Click(object sender, RoutedEventArgs e)
        {
            clearAllButton(BnText, CADParent.GrfTypen.C_TEXT);
        }
        


        private void bnLineColorLine_Click(object sender, RoutedEventArgs e)
        {
            // Labor2
            ColorDialog dialog = new ColorDialog();
            dialog.setColor(colorLine);
            dialog.ShowDialog();
            if (dialog.Retcode)
            {
                colorLine = Color.FromArgb(dialog.Alpha, dialog.Red, dialog.Green, dialog.Blue);
                bnLineColor.Background = new SolidColorBrush(colorLine);
            }
        }

        private void bnFillColorLine_Click(object sender, RoutedEventArgs e)
        {
            // Labor2
            ColorDialog dialog = new ColorDialog();
            dialog.setColor(colorFill);
            dialog.ShowDialog();
            if (dialog.Retcode)
            {
                colorFill = Color.FromArgb(dialog.Alpha, dialog.Red, dialog.Green, dialog.Blue);
                bnFillColor.Background = new SolidColorBrush(colorFill);
            }

        }


        private void NewExecuted(object sender, ExecutedRoutedEventArgs e)
        {
            MessageBox.Show("NewExecuted");
        }



        private void Export2DXFfile(object sender, ExecutedRoutedEventArgs e)
        {
            SaveFileDialog saveFileDialog1 = new SaveFileDialog();
            saveFileDialog1.Filter =
               "DXF Dateien (*.dxf)|*.dxf|Alle Dateien (*.*)|*.*";
            saveFileDialog1.FilterIndex = 1;// zählt von 1 !
            saveFileDialog1.DefaultExt = ".dxf";
            saveFileDialog1.InitialDirectory = System.Environment.CurrentDirectory;
            saveFileDialog1.RestoreDirectory = true;
            if (saveFileDialog1.ShowDialog() == true)
            {
                // Verarbeitung
                string sFile = saveFileDialog1.FileName.ToString();

                Middleware.getInstance().export2DXF(sFile);
            }
        }






        // Labor2
        private void BnUpGraphic_Click(object sender, RoutedEventArgs e)
        {
            int i = graphicList.SelectedIndex;
            if ((Middleware.getInstance().cadListe.Count > 0) && (i > 0))
            {
                Middleware.getInstance().upGraphic(i);
                CADParent grf = (CADParent)graphicList.Items[i];
                graphicList.Items.RemoveAt(i);
                graphicList.Items.Insert(i - 1, grf);
                graphicList.SelectedIndex = -1;
                graphicList.SelectedIndex = i - 1;
                canvas.InvalidateVisual();  // startet onRender
            }
        }
        // Labor2
        private void BnDownGraphic_Click(object sender, RoutedEventArgs e)
        {
            int i = graphicList.SelectedIndex;
            int n = Middleware.getInstance().cadListe.Count;
            if ((n > 0) && (i >= 0) && (i < (n - 1)))
            {
                Middleware.getInstance().downGraphic(i);
                CADParent grf = (CADParent)graphicList.Items[i];
                graphicList.Items.RemoveAt(i);
                graphicList.Items.Insert(i + 1, grf);
                graphicList.SelectedIndex = -1;
                graphicList.SelectedIndex = i + 1;
                canvas.InvalidateVisual();  // startet onRender
            }
        }
        // Labor2
        private void DeleteElement(object sender, ExecutedRoutedEventArgs e)
        {
            int i = graphicList.SelectedIndex;
            if ((Middleware.getInstance().cadListe.Count > 0) && (i >= 0))
            {
                Middleware.getInstance().deleteGraphic(i);
                graphicList.Items.RemoveAt(i);
                canvas.InvalidateVisual();  // startet onRender
            }
        }





        // Labor3
        public void setCoordX(double value)
        {
            double dFactorMM = 94.5 / 25.4;  // Pixel pro mm
            coordX.Content = "x: " + (value / dFactorMM).ToString("0.0") + " cm";
        }

        // Labor3
        public void setCoordY(double value)
        {
            double dFactorMM = 94.5 / 25.4;  // Pixel pro mm
            coordY.Content = "y: " + (value / dFactorMM).ToString("0.0") + " cm";
        }


        // Labor3
        public void insertGraphicsText(double zoom, double x, double y, double baseline)
        {
            InputText dialog = new InputText();
            dialog.ShowDialog();
            if (dialog.Retcode)
            {
               string sText = dialog.Text.Trim();
               Middleware.getInstance().insertGraphicText(zoom, x, y, baseline, sText, colorLine, colorFill, graphicList);
               clearAllButton(BnArrowNone, CADParent.GrfTypen.C_NONE);
               canvas.Modus = CADParent.GrfTypen.C_NONE;
               canvas.InvalidateVisual();  // startet onRender
            }
            else
            {
               clearAllButton(BnArrowNone, CADParent.GrfTypen.C_NONE);
               canvas.Modus = CADParent.GrfTypen.C_NONE;
            }
        }  // insertGraphicsText


        public void insertGraphic(double zoom, double mx1, double my1, double mx2, double my2)
        {
            double dFactorMM = 94.5 / 25.4;  // Pixel pro mm

            int x1, x2, y1, y2;
            double f = dFactorMM*zoom;
            x1 = (int) Math.Round(mx1 / f);
            y1 = (int) Math.Round(my1 / f);
            x2 = (int) Math.Round(mx2 / f);
            y2 = (int) Math.Round(my2 / f);            

            switch (canvas.Modus)
            {
                case CADParent.GrfTypen.C_LINE:
                case CADParent.GrfTypen.C_HORILINE:
                case CADParent.GrfTypen.C_VERTLINE:
                    CADLine line1;
                            line1 = new CADLine(x1, y1, x2, y2);
                            line1.StrokeThickness = linewidth;
                            line1.LineStyle = lineStyle;

                            line1.BrushLine =  colorLine;  
                            Middleware.getInstance().cadListe.Add(line1);
                            graphicList.Items.Add(line1);
                            break;

                case CADParent.GrfTypen.C_RECTANGLE:
                            CADRectangle rect;
                            rect = new CADRectangle(x1, y1, x2, y2);
                            rect.StrokeThickness = 15;
                            rect.brushLine = colorLine;
                            rect.brushFill = colorFill;
                            Middleware.getInstance().cadListe.Add(rect);
                            graphicList.Items.Add(rect);
                            break;
                case CADParent.GrfTypen.C_CIRCLE:
                            CADCircle circle;
                            circle = new CADCircle(x1, y1, x2, y2);
                            circle.StrokeThickness = 15;
                            circle.brushLine = colorLine;
                            circle.brushFill = colorFill;
                            Middleware.getInstance().cadListe.Add(circle);
                            graphicList.Items.Add(circle);
                            break;
                case CADParent.GrfTypen.C_ELLIPSE:
                            CADEllipse ellipse;
                            ellipse = new CADEllipse(x1, y1, x2, y2);
                            ellipse.StrokeThickness = 15;
                            ellipse.brushLine = colorLine;
                            ellipse.brushFill = colorFill;
                            Middleware.getInstance().cadListe.Add(ellipse);
                            graphicList.Items.Add(ellipse);
                            break;

            }  // case
        }  // insertGraphic



        // Labor 4
        private void BnEditGraphic_Click(object sender, RoutedEventArgs e)
        {
            editElement();
        }

        // Labor 4
        private void graphicList_MouseDoubleClick(
                                  object sender, MouseButtonEventArgs e)
        {
            editElement();
        }

        // Labor 4
        private void editElement()
        {
            //  editieren
            int i = graphicList.SelectedIndex;

            if ((Middleware.getInstance().cadListe.Count > 0) && (i >= 0))
            {
                CADParent grf = (CADParent)graphicList.Items[i];
                switch (grf.GraphicTyp)
                {
                    case CADParent.GrfTypen.C_LINE:
                        EditLine dialog1 = new EditLine((CADLine)grf);
                        dialog1.ShowDialog();
                        if (dialog1.Retcode)
                        {
                            canvas.InvalidateVisual();  // startet onRender
                        }

                        break;

                    case CADParent.GrfTypen.C_RECTANGLE:                        
                        EditRectangle dialog2 = new EditRectangle( (CADRectangle)grf);
                        dialog2.ShowDialog();
                        if (dialog2.Retcode)
                        {
                            canvas.InvalidateVisual();  // startet onRender
                        }
                        break;

                    case CADParent.GrfTypen.C_CIRCLE:
                        EditCircle dialog3 = new EditCircle((CADCircle)grf );
                        dialog3.ShowDialog();
                        if (dialog3.Retcode)
                        {
                            canvas.InvalidateVisual();  // startet onRender
                        }
                        break;

                    case CADParent.GrfTypen.C_ELLIPSE:
                        EditEllipse dialog4 = new EditEllipse((CADEllipse)grf);
                        dialog4.ShowDialog();
                        if (dialog4.Retcode)
                        {
                            canvas.InvalidateVisual();  // startet onRender
                        }
                        break;

                    case CADParent.GrfTypen.C_TEXT:
                        EditText dialog5 = new EditText((CADText)grf);
                        dialog5.ShowDialog();
                        if (dialog5.Retcode)
                        {
                            canvas.InvalidateVisual();  // startet onRender
                        }
                        break;
                    default: 
                        MessageBox.Show("Fehlerhafter switch-case in editElement, index: " + grf.GraphicTyp, "Fehler");
                        break;
                }  // switch

            }
        } // editElement


        // Labor 4
        private void CopyElement(object sender, ExecutedRoutedEventArgs e)
        {
            int i = graphicList.SelectedIndex;
            if ((Middleware.getInstance().cadListe.Count > 0) && (i >= 0))
            {
                _copyPasteElement = (CADParent)graphicList.Items[i];
            }
        }
        // Labor 4
        private void PasteElement(object sender, ExecutedRoutedEventArgs e)
        {
            if (_copyPasteElement == null) return;
            //MessageBox.Show("error");
            switch (_copyPasteElement.GraphicTyp)
            {
                case CADParent.GrfTypen.C_LINE:
                    CADLine line = (CADLine)_copyPasteElement;
                    CADLine line2 = new CADLine(line);
                    line2.X1 += 20;
                    line2.X2 += 20;
                    Middleware.getInstance().cadListe.Add(line2);
                    graphicList.Items.Add(line2);
                    canvas.InvalidateVisual();  // startet onRender
                    break;

                case CADParent.GrfTypen.C_RECTANGLE:
                    CADRectangle rect = (CADRectangle)_copyPasteElement;
                    CADRectangle rect2 = new CADRectangle(rect);
                    rect2.X1 += 20;
                    rect2.X2 += 20;
                    Middleware.getInstance().cadListe.Add(rect2);
                    graphicList.Items.Add(rect2);
                    canvas.InvalidateVisual();  // startet onRender
                    break;

                case CADParent.GrfTypen.C_CIRCLE:
                    CADCircle circle = (CADCircle)_copyPasteElement;
                    CADCircle circle2 = new CADCircle(circle);
                    circle2.X += 20;
                    circle2.X += 20;
                    Middleware.getInstance().cadListe.Add(circle2);
                    graphicList.Items.Add(circle2);
                    canvas.InvalidateVisual();  // startet onRender
                    break;

                case CADParent.GrfTypen.C_ELLIPSE:
                    CADEllipse ellipse = (CADEllipse)_copyPasteElement;
                    CADEllipse ellipse2 = new CADEllipse(ellipse);
                    ellipse2.X += 20;
                    ellipse2.X += 20;
                    Middleware.getInstance().cadListe.Add(ellipse2);
                    graphicList.Items.Add(ellipse2);
                    canvas.InvalidateVisual();  // startet onRender
                    break;

                case CADParent.GrfTypen.C_TEXT:
                    CADText text = (CADText)_copyPasteElement;
                    CADText text2 = new CADText(text);
                    text2.X += 20;
                    text2.X += 20;
                    Middleware.getInstance().cadListe.Add(text2);
                    graphicList.Items.Add(text2);
                    canvas.InvalidateVisual();  // startet onRender
                    break;


            }  // switch
            // Middleware.getInstance().cadListe.Add(
            // graphicList.Items.RemoveAt(i);
            canvas.InvalidateVisual();  // startet onRender
        }  // Paste

        // Labor 4
        private void OpenExecuted(object sender, ExecutedRoutedEventArgs e)
        {
            OpenFileDialog openFileDialog1 = new OpenFileDialog();
            openFileDialog1.Filter =
               "CAD-XML Dateien (*.xml)|*.xml|Alle Dateien (*.*)|*.*";
            openFileDialog1.FilterIndex = 1;// zählt von 1 !
            openFileDialog1.DefaultExt = ".xml";
            openFileDialog1.InitialDirectory = System.Environment.CurrentDirectory;
            openFileDialog1.Multiselect = false;  // true
            openFileDialog1.RestoreDirectory = true;
            if (openFileDialog1.ShowDialog() == true)
            {
                // Verarbeitung
                string sFile = openFileDialog1.FileName;
                Title = sFile;                
                Middleware.getInstance().loadlisteXml(sFile);
                graphicList.Items.Clear();
                foreach (CADParent grf in Middleware.getInstance().cadListe)
                {
                    graphicList.Items.Add(grf);
                }
                canvas.InvalidateVisual();  // startet onRender
            }
        }


        // Labor 4
        private void SaveExecuted(object sender, ExecutedRoutedEventArgs e)
        {
            SaveFileDialog saveFileDialog1 = new SaveFileDialog();
            saveFileDialog1.Filter =
               "CAD-XML Dateien (*.xml)|*.xml|Alle Dateien (*.*)|*.*";
            saveFileDialog1.FilterIndex = 1;// zählt von 1 !
            saveFileDialog1.DefaultExt = ".xml";
            saveFileDialog1.InitialDirectory = System.Environment.CurrentDirectory;
            saveFileDialog1.RestoreDirectory = true;
            if (saveFileDialog1.ShowDialog() == true)
            {
                // Verarbeitung
                string sFile = saveFileDialog1.FileName.ToString();
                Title = sFile;
                Middleware.getInstance().savelisteXml(sFile);
            }
        }


        // Labor 4
        private void PrintElements(object sender, ExecutedRoutedEventArgs e)
        {
            int i = graphicList.SelectedIndex;
            if (Middleware.getInstance().cadListe.Count > 0)
            {

                SaveFileDialog saveFileDialog1 = new SaveFileDialog();
                saveFileDialog1.Filter =
                   "XPS Datei (*.xps)|*.xps|Alle Dateien (*.*)|*.*";
                saveFileDialog1.FilterIndex = 1;// zählt von 1 !
                saveFileDialog1.DefaultExt = ".xps";
                saveFileDialog1.InitialDirectory = System.Environment.CurrentDirectory;
                saveFileDialog1.RestoreDirectory = true;
                if (saveFileDialog1.ShowDialog() == true)
                {
                    // Verarbeitung
                    string sFilename = saveFileDialog1.FileName.ToString();

                    // FixedDocSeq und FixedDocument erstellen
                    FixedDocumentSequence fSeq = new FixedDocumentSequence();
                    FixedDocument fDoc = new FixedDocument();

                    // DocumentReference mit FixedDoc zu FixedDocSeq hinzufügen
                    DocumentReference docRef = new DocumentReference();
                    docRef.SetDocument(fDoc);
                    fSeq.References.Add(docRef);

                    // Fixed Page erstellen
                    FixedPage fPage = new FixedPage();
                    fPage.Width = 816;
                    fPage.Height = 1056;

                    //TextBlock txt = new TextBlock();
                    //txt.Text = "Drucken mit CAD";
                    //fPage.Children.Add(txt);
                    
                    MyCanvas canvas2 = new MyCanvas(this);
                    canvas2.Background = Brushes.White;
                    fPage.Children.Add(canvas2);
                    
                    // FixedPage zu PageContent hinzufügen
                    PageContent pageContent = new PageContent();
                    ((IAddChild)pageContent).AddChild(fPage);

                    // PageContent zu FixedDoc hinzufügen
                    fDoc.Pages.Add(pageContent);

                    // Als XPS speichern

                    if (File.Exists(sFilename))
                    {
                        File.Delete(sFilename);
                    }
                    XpsDocument doc = new XpsDocument(sFilename, FileAccess.Write);
                    XpsDocumentWriter writer = XpsDocument.CreateXpsDocumentWriter(doc);
                    writer.Write(fSeq);
                    doc.Close();

                    Process.Start(sFilename);

                } // if savedialog

            }  // if liste not null
        } // PrintElements


        // Labor 4
        private void InsertPerConsole(object sender, ExecutedRoutedEventArgs e)
        {
            ConsoleDialog dialog = new ConsoleDialog(this);
            dialog.Show();
        }


        // Labor 4
        public void insertCADParent(CADParent element)
        {
            Middleware.getInstance().cadListe.Add(element);
            graphicList.Items.Add(element);
            canvas.InvalidateVisual();  // startet onRender
        }

        // Labor 4
        public void deleteElements()
        {
            Middleware.getInstance().cadListe.Clear();
            graphicList.Items.Clear();
            canvas.InvalidateVisual();  // startet onRender
        }


        // Labor4
        private void tInputLinewidth_TextChanged(object sender, TextChangedEventArgs e)
        {
            double linewidth;
            string sValue = tInputLinewidth.Text.Trim();
            if (double.TryParse(sValue, out linewidth))
            {
                if (linewidth > 0.0) this.linewidth = linewidth;
            }

        }


        // Labor4
        private void Linestyle_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            if (bStart) return;
            switch (cbLinestyle.SelectedIndex)
            {
                case 0: lineStyle = DashStyles.Solid;
                    break;
                case 1: lineStyle = DashStyles.Dash;
                    break;
                case 2: lineStyle = DashStyles.DashDot;
                    break;
                case 3: lineStyle = DashStyles.DashDotDot;
                    break;
                case 4: lineStyle = DashStyles.Dot;
                    break;
            }
        }

        // Labor4
        private void cbZoom_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            if (bStart) return;
            int zoom;
            ComboBoxItem item = (ComboBoxItem)cbZoom.Items[cbZoom.SelectedIndex];  // 125%
            string sValue = (string)item.Tag;
            if (int.TryParse(sValue, out zoom))
            {
                canvas.Zoom = zoom * 0.01;
            }

        } 


    }
}
